import * as i0 from '@angular/core';
import { Component, ViewEncapsulation, Host, Input, Injectable, Directive, HostListener, NgModule } from '@angular/core';
import { Subject } from 'rxjs';
import { finalize } from 'rxjs/operators';
import { CommonModule } from '@angular/common';

class TooltipComponent {
    constructor(_host) {
        this._host = _host;
        this.content = '';
        this.placement = 'bottom';
        /**
         * @private To be used by template
         */
        this._isActive = false;
        /**
         * @private To be used by template
         */
        this._idForAriaDescribedBy = `__${btoa(String(Math.random() + Math.random())).substring(5, 15).toLowerCase()}__`;
        /**
         * Width of the tooltip
         */
        this._width = 0;
        /**
         * Height of the tooltip
         */
        this._height = 0;
    }
    ngAfterViewInit() {
        if (!['top', 'right', 'bottom', 'left'].includes(this.placement)) {
            throw new Error('Invalid tooltip placement: ' + this.placement);
        }
        this._tooltip = this._host.nativeElement.firstElementChild;
        const tooltipRect = this._tooltip.getBoundingClientRect();
        this._width = tooltipRect.width;
        this._height = tooltipRect.height;
    }
    hide() {
        this._isActive = false;
    }
    /**
     * Execute the provide callback after this tooltip is hidden
     *
     * @param fn
     */
    runAfterClosed(fn) {
        this._afterClosed = fn;
    }
    /**
     * Show a tooltip at the `anchor` element
     *
     * @param anchor
     */
    show(anchor) {
        this._isActive = true;
        anchor.setAttribute('aria-describedby', this._idForAriaDescribedBy);
        switch (this.placement) {
            case 'top':
                this._showTop(anchor.getBoundingClientRect());
                break;
            case 'right':
                this._showRight(anchor.getBoundingClientRect());
                break;
            case 'bottom':
                this._showBottom(anchor.getBoundingClientRect());
                break;
            case 'left':
                this._showLeft(anchor.getBoundingClientRect());
                break;
        }
    }
    /**
     * Show a tooltip at the spefified x/y location
     *
     * @param x
     * @param y
     */
    showAt(x, y) {
        this._isActive = true;
        const anchorBoundingBox = {
            left: x,
            top: y,
            width: 0,
            height: 0
        };
        switch (this.placement) {
            case 'top':
                this._showTop(anchorBoundingBox);
                break;
            case 'right':
                this._showRight(anchorBoundingBox);
                break;
            case 'bottom':
                this._showBottom(anchorBoundingBox);
                break;
            case 'left':
                this._showLeft(anchorBoundingBox);
                break;
        }
    }
    /**
     *
     * @private Used in template, so can't have `private` keyword
     */
    _onLeave() {
        if (!this._isActive && this._afterClosed) {
            this._afterClosed();
        }
    }
    _showTop(anchorBoundingBox) {
        setTimeout(() => {
            const left = anchorBoundingBox.left + (anchorBoundingBox.width - this._width) / 2;
            let top = anchorBoundingBox.top - this._height - 10;
            if (top < 0) {
                this._tooltip.classList.remove('top');
                this._tooltip.classList.add('bottom');
                top = top + anchorBoundingBox.height + this._height + 20;
            }
            if (!this._shiftTooltipIntoViewFromRight(left)) {
                this._shiftTooltipIntoViewFromLeft(left);
            }
            this._tooltip.style.left = left + 'px';
            this._tooltip.style.top = top + 'px';
        }, 0);
    }
    /**
     * Returns true if tooltip overflows the right edge of the screen, false otherwise
     */
    _shiftTooltipIntoViewFromRight(left) {
        const difference = left + this._width - window.innerWidth + 5;
        if (difference > 5) {
            this._tooltip.style.transform = `translateX(-${difference}px)`;
            this._tooltip.querySelector('.kun-tooltip__arrow').style.left = `calc(50% + ${difference}px)`;
            return true;
        }
        return false;
    }
    /**
     *
     * Returns true if tooltip overflows the left edge of the screen, false otherwise
     */
    _shiftTooltipIntoViewFromLeft(left) {
        if (left < 0) {
            left = Math.abs(left) + 5;
            this._tooltip.style.transform = `translateX(${left}px)`;
            this._tooltip.querySelector('.kun-tooltip__arrow').style.left = `calc(50% - ${left}px)`;
            return true;
        }
        return false;
    }
    _showBottom(anchorBoundingBox) {
        setTimeout(() => {
            const left = anchorBoundingBox.left + (anchorBoundingBox.width - this._width) / 2;
            let top = anchorBoundingBox.top + anchorBoundingBox.height + 10;
            if (top + this._height + 10 > window.innerHeight) {
                this._tooltip.classList.remove('bottom');
                this._tooltip.classList.add('top');
                top = top - anchorBoundingBox.height - this._height - 20;
            }
            if (!this._shiftTooltipIntoViewFromRight(left)) {
                this._shiftTooltipIntoViewFromLeft(left);
            }
            this._tooltip.style.left = left + 'px';
            this._tooltip.style.top = top + 'px';
        }, 0);
    }
    _showRight(anchorBoundingBox) {
        setTimeout(() => {
            let left = anchorBoundingBox.left + anchorBoundingBox.width + 10;
            const top = anchorBoundingBox.top + (anchorBoundingBox.height - this._height) / 2;
            if (left + this._width > window.innerWidth) {
                this._tooltip.classList.remove('right');
                this._tooltip.classList.add('left');
                left = left - anchorBoundingBox.width - this._width - 20;
            }
            if (!this._shiftTooltipIntoViewFromBottom(top)) {
                this._shiftTooltipIntoViewFromTop(top);
            }
            this._tooltip.style.left = left + 'px';
            this._tooltip.style.top = top + 'px';
        }, 0);
    }
    /**
     *
     * @param top Top edge value of the tooltip
     * @returns `true` if tooltip overflows the bottom edge of the screen, `false` otherwise
     */
    _shiftTooltipIntoViewFromBottom(top) {
        const difference = top + this._height - window.innerHeight + 5;
        if (difference > 5) {
            this._tooltip.style.transform = `translateY(-${difference}px)`;
            this._tooltip.querySelector('.kun-tooltip__arrow').style.top = `calc(50% + ${difference}px)`;
            return true;
        }
        return false;
    }
    /**
     *
     * @param top Top edge value of the tooltip
     * @returns `true` if tooltip overflows the bottom top of the screen, `false` otherwise
     */
    _shiftTooltipIntoViewFromTop(top) {
        if (top < 0) {
            top = Math.abs(top) + 5;
            this._tooltip.style.transform = `translateY(${top}px)`;
            this._tooltip.querySelector('.kun-tooltip__arrow').style.top = `calc(50% - ${top}px)`;
            return true;
        }
        return false;
    }
    _showLeft(anchorBoundingBox) {
        /**
         * Wrap this in a `setTimeout` because tooltip is animated, so by the time `getBoundingClientRect()`
         * is called, tooltip component might not be visible yet, so it can't calculate tooltip's client rect
         */
        setTimeout(() => {
            let left = anchorBoundingBox.left - this._width - 10;
            const top = anchorBoundingBox.top + (anchorBoundingBox.height - this._height) / 2;
            if (left < 0) {
                this._tooltip.classList.remove('left');
                this._tooltip.classList.add('right');
                left = left + anchorBoundingBox.width + this._width + 20;
            }
            if (!this._shiftTooltipIntoViewFromBottom(top)) {
                this._shiftTooltipIntoViewFromTop(top);
            }
            this._tooltip.style.left = left + 'px';
            this._tooltip.style.top = top + 'px';
        }, 0);
    }
}
TooltipComponent.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.2.5", ngImport: i0, type: TooltipComponent, deps: [{ token: i0.ElementRef, host: true }], target: i0.ɵɵFactoryTarget.Component });
TooltipComponent.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "12.0.0", version: "13.2.5", type: TooltipComponent, selector: "kun-tooltip", inputs: { content: "content", placement: "placement" }, host: { classAttribute: "kun-tooltip-container" }, ngImport: i0, template: "<div class='kun-tooltip {{placement}} {{_isActive ? \"enter\": \"leave\"}}'\r\n     (animationend)='_onLeave()'>\r\n     <div class='kun-tooltip__arrow'></div>\r\n     <div class='kun-tooltip__content'\r\n          [id]='_idForAriaDescribedBy'\r\n          [innerHTML]='content'>\r\n     </div>\r\n</div>", styles: [".kun-tooltip-container{position:fixed;display:block;top:0;left:0;bottom:0;right:0;z-index:1000;pointer-events:none!important}.kun-tooltip{position:absolute;border-radius:3px;z-index:15;white-space:pre-wrap;font-family:Roboto,sans-serif;font-size:.8em;max-width:99vw;box-shadow:0 0 2px #271e1e;background-color:#555;color:#fff;border:1px solid #8e8e8e;animation:.25s cubic-bezier(.04,.54,.25,1) forwards}.kun-tooltip.enter{animation-name:enter}.kun-tooltip.leave{animation-name:leave}@keyframes enter{0%{opacity:0;visibility:hidden}to{opacity:1;visibility:visible}}@keyframes leave{0%{opacity:1;visibility:visible}to{opacity:0;visibility:hidden}}.kun-tooltip__arrow{overflow:hidden;position:absolute;width:20px;height:20px}.kun-tooltip__arrow:after{content:\"\";position:relative;width:inherit;height:inherit;display:inline-block;background-color:#555;border:1px solid rgba(0,0,0,.1);box-shadow:0 0 2px #271e1e;border:1px solid #8e8e8e;border-radius:3px}.kun-tooltip__content{line-height:1.5em;padding:10px;max-width:99vw}.kun-tooltip.top .kun-tooltip__arrow{transform:translate(-50%);top:100%;left:50%}.kun-tooltip.top .kun-tooltip__arrow:after{transform:translateY(-19px) rotate(45deg)}.kun-tooltip.right .kun-tooltip__arrow{transform:translateY(-55%) translate(-20px);top:50%;left:0}.kun-tooltip.right .kun-tooltip__arrow:after{transform:translate(75%) rotate(45deg)}.kun-tooltip.bottom .kun-tooltip__arrow{transform:translate(-50%);top:-20px;left:50%}.kun-tooltip.bottom .kun-tooltip__arrow:after{transform:translateY(15px) rotate(45deg)}.kun-tooltip.left .kun-tooltip__arrow{transform:translateY(-55%);top:50%;left:100%}.kun-tooltip.left .kun-tooltip__arrow:after{transform:translate(-80%) rotate(45deg)}\n"], encapsulation: i0.ViewEncapsulation.None });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.2.5", ngImport: i0, type: TooltipComponent, decorators: [{
            type: Component,
            args: [{ selector: 'kun-tooltip', encapsulation: ViewEncapsulation.None, host: {
                        class: 'kun-tooltip-container'
                    }, template: "<div class='kun-tooltip {{placement}} {{_isActive ? \"enter\": \"leave\"}}'\r\n     (animationend)='_onLeave()'>\r\n     <div class='kun-tooltip__arrow'></div>\r\n     <div class='kun-tooltip__content'\r\n          [id]='_idForAriaDescribedBy'\r\n          [innerHTML]='content'>\r\n     </div>\r\n</div>", styles: [".kun-tooltip-container{position:fixed;display:block;top:0;left:0;bottom:0;right:0;z-index:1000;pointer-events:none!important}.kun-tooltip{position:absolute;border-radius:3px;z-index:15;white-space:pre-wrap;font-family:Roboto,sans-serif;font-size:.8em;max-width:99vw;box-shadow:0 0 2px #271e1e;background-color:#555;color:#fff;border:1px solid #8e8e8e;animation:.25s cubic-bezier(.04,.54,.25,1) forwards}.kun-tooltip.enter{animation-name:enter}.kun-tooltip.leave{animation-name:leave}@keyframes enter{0%{opacity:0;visibility:hidden}to{opacity:1;visibility:visible}}@keyframes leave{0%{opacity:1;visibility:visible}to{opacity:0;visibility:hidden}}.kun-tooltip__arrow{overflow:hidden;position:absolute;width:20px;height:20px}.kun-tooltip__arrow:after{content:\"\";position:relative;width:inherit;height:inherit;display:inline-block;background-color:#555;border:1px solid rgba(0,0,0,.1);box-shadow:0 0 2px #271e1e;border:1px solid #8e8e8e;border-radius:3px}.kun-tooltip__content{line-height:1.5em;padding:10px;max-width:99vw}.kun-tooltip.top .kun-tooltip__arrow{transform:translate(-50%);top:100%;left:50%}.kun-tooltip.top .kun-tooltip__arrow:after{transform:translateY(-19px) rotate(45deg)}.kun-tooltip.right .kun-tooltip__arrow{transform:translateY(-55%) translate(-20px);top:50%;left:0}.kun-tooltip.right .kun-tooltip__arrow:after{transform:translate(75%) rotate(45deg)}.kun-tooltip.bottom .kun-tooltip__arrow{transform:translate(-50%);top:-20px;left:50%}.kun-tooltip.bottom .kun-tooltip__arrow:after{transform:translateY(15px) rotate(45deg)}.kun-tooltip.left .kun-tooltip__arrow{transform:translateY(-55%);top:50%;left:100%}.kun-tooltip.left .kun-tooltip__arrow:after{transform:translate(-80%) rotate(45deg)}\n"] }]
        }], ctorParameters: function () {
        return [{ type: i0.ElementRef, decorators: [{
                        type: Host
                    }] }];
    }, propDecorators: { content: [{
                type: Input
            }], placement: [{
                type: Input
            }] } });

/**
 * This class allows to programmatically show tooltip anchored at an element or at a specific x/y location
 */
class TooltipService {
    constructor(_injector, _applicationRef, componentFactoryResolver) {
        this._injector = _injector;
        this._applicationRef = _applicationRef;
        this._tooltipRef = null;
        this._tooltipComponentFactory = componentFactoryResolver.resolveComponentFactory(TooltipComponent);
    }
    /**
     * Programmatically show tooltip anchored at the element specified by `target`
     *
     * @param target The element at which to place the tooltip.
     * @param content Tooltip's body's content, it can contain HTML
     * @param placement Where the tooltip should be anchored at, `bottom` is the default
     */
    show(target, content, placement = 'bottom') {
        this._tooltipRef = this._createTooltip(content, placement);
        this._mountTooltip(this._tooltipRef);
        this._tooltipRef.instance.show(target);
    }
    /**
     * A helper method used to create new tooltip component refs
     *
     * @param content Tooltip's body's content, it can contain HTML
     * @param placement Where the tooltip should be anchored at
     * @returns
     */
    _createTooltip(content, placement) {
        const componentRef = this._tooltipComponentFactory.create(this._injector);
        componentRef.instance.content = content;
        componentRef.instance.placement = placement;
        componentRef.instance.runAfterClosed(() => {
            this._applicationRef.detachView(componentRef.hostView);
            componentRef.hostView.destroy();
        });
        return componentRef;
    }
    /**
     * Mount `tooltipRef` and make its host element a direct child of `document.body`
     *
     * @param tooltipRef
     */
    _mountTooltip(tooltipRef) {
        this._applicationRef.attachView(tooltipRef.hostView);
        document.body.appendChild(tooltipRef.location.nativeElement);
    }
    /**
     * Show tooltip at a specific x/y location
     *
     * @param x The x location
     * @param y The y location
     * @param content Tooltip's body's content, it can contain HTML
     * @param placement Where the tooltip should be anchored at, `bottom` is the default
     */
    showAt(x, y, content, placement = 'bottom') {
        this._tooltipRef = this._createTooltip(content, placement);
        this._mountTooltip(this._tooltipRef);
        this._tooltipRef.instance.showAt(x, y);
    }
    /**
     * Hide the currently active tooltip
     */
    hide() {
        if (this._tooltipRef) {
            this._tooltipRef.instance.hide();
            this._tooltipRef = null;
        }
    }
}
TooltipService.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.2.5", ngImport: i0, type: TooltipService, deps: [{ token: i0.Injector }, { token: i0.ApplicationRef }, { token: i0.ComponentFactoryResolver }], target: i0.ɵɵFactoryTarget.Injectable });
TooltipService.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "13.2.5", ngImport: i0, type: TooltipService, providedIn: 'root' });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.2.5", ngImport: i0, type: TooltipService, decorators: [{
            type: Injectable,
            args: [{
                    providedIn: 'root'
                }]
        }], ctorParameters: function () { return [{ type: i0.Injector }, { type: i0.ApplicationRef }, { type: i0.ComponentFactoryResolver }]; } });

function isMobile() {
    return /iPhone|iPad|Android/i.test(navigator.userAgent);
}
function watchForLongPress(target) {
    let timeoutId;
    const notifier = new Subject();
    const pointerdown = (event) => {
        timeoutId = window.setTimeout(() => notifier.next(event), 500);
    };
    const pointerup = () => {
        window.clearTimeout(timeoutId);
    };
    target.addEventListener('pointerdown', pointerdown, false);
    window.addEventListener('pointerup', pointerup, false);
    target.oncontextmenu = event => {
        event.preventDefault();
    };
    return notifier.pipe(finalize(() => {
        target.removeEventListener('pointerdown', pointerdown, false);
        window.removeEventListener('pointerup', pointerup, false);
    }));
}

class TooltipDirective {
    constructor(_hostElement, _tooltipService) {
        this._hostElement = _hostElement;
        this._tooltipService = _tooltipService;
        this.content = '';
        this.placement = 'bottom';
    }
    ngOnDestroy() {
        var _a;
        this._tooltipService.hide();
        (_a = this._longPressEventSubscription) === null || _a === void 0 ? void 0 : _a.unsubscribe();
    }
    ngAfterViewInit() {
        if (isMobile()) {
            this._longPressEventSubscription = watchForLongPress(this._hostElement.nativeElement)
                .subscribe({
                next: event => {
                    this._showTooltip(event);
                    setTimeout(() => this._hideTooltip(event), 5000);
                }
            });
        }
    }
    _showTooltip(event) {
        this._hideTooltip(event);
        this._timer = window.setTimeout(() => {
            event.stopPropagation();
            this._tooltipService.show(event.target, this.content, this.placement);
        }, 250);
    }
    _hideTooltip(event) {
        event.stopPropagation();
        window.clearTimeout(this._timer);
        this._tooltipService.hide();
    }
    /**
     * Tooltip is only shown on mouseover events iff we are not on a mobile device
     */
    _onShowTooltipOnHover(event) {
        if (!isMobile()) {
            this._showTooltip(event);
        }
    }
    _onShowTooltipOnFocus(event) {
        if (document.activeElement === this._hostElement.nativeElement) {
            this._showTooltip(event);
        }
    }
    _onHideTooltip(event) {
        this._hideTooltip(event);
    }
}
TooltipDirective.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.2.5", ngImport: i0, type: TooltipDirective, deps: [{ token: i0.ElementRef, host: true }, { token: TooltipService }], target: i0.ɵɵFactoryTarget.Directive });
TooltipDirective.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "12.0.0", version: "13.2.5", type: TooltipDirective, selector: "[kunTooltip]", inputs: { content: ["kunTooltip", "content"], placement: ["kunTooltipPlacement", "placement"] }, host: { listeners: { "mouseover": "_onShowTooltipOnHover($event)", "keyup": "_onShowTooltipOnFocus($event)", "mouseout": "_onHideTooltip($event)", "blur": "_onHideTooltip($event)" } }, ngImport: i0 });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.2.5", ngImport: i0, type: TooltipDirective, decorators: [{
            type: Directive,
            args: [{
                    selector: '[kunTooltip]'
                }]
        }], ctorParameters: function () {
        return [{ type: i0.ElementRef, decorators: [{
                        type: Host
                    }] }, { type: TooltipService }];
    }, propDecorators: { content: [{
                type: Input,
                args: ['kunTooltip']
            }], placement: [{
                type: Input,
                args: ['kunTooltipPlacement']
            }], _onShowTooltipOnHover: [{
                type: HostListener,
                args: ['mouseover', ['$event']]
            }], _onShowTooltipOnFocus: [{
                type: HostListener,
                args: ['keyup', ['$event']]
            }], _onHideTooltip: [{
                type: HostListener,
                args: ['mouseout', ['$event']]
            }, {
                type: HostListener,
                args: ['blur', ['$event']]
            }] } });

class TooltipModule {
}
TooltipModule.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.2.5", ngImport: i0, type: TooltipModule, deps: [], target: i0.ɵɵFactoryTarget.NgModule });
TooltipModule.ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "12.0.0", version: "13.2.5", ngImport: i0, type: TooltipModule, declarations: [TooltipComponent, TooltipDirective], imports: [CommonModule], exports: [TooltipDirective] });
TooltipModule.ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "13.2.5", ngImport: i0, type: TooltipModule, imports: [[
            CommonModule
        ]] });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.2.5", ngImport: i0, type: TooltipModule, decorators: [{
            type: NgModule,
            args: [{
                    declarations: [TooltipComponent, TooltipDirective],
                    imports: [
                        CommonModule
                    ],
                    exports: [
                        TooltipDirective
                    ]
                }]
        }] });

/**
 * Generated bundle index. Do not edit.
 */

export { TooltipDirective, TooltipModule, TooltipService };
//# sourceMappingURL=angular-simple-tooltip.mjs.map
